@@ -196,10 +196,16 @@ module Pantry
   ) where
 
 import           Casa.Client ( CasaRepoPrefix, thParserCasaRepo )
-import           Conduit
+import           Conduit ( (.|), mapC, mapMC, runConduitRes, sinkList, sumC )
+import           Control.Applicative ( empty )
 import           Control.Arrow ( right )
 import           Control.Monad.State.Strict ( State, execState, get, modify' )
-import           Data.Aeson.Types ( parseEither )
+import           Control.Monad.Trans.Maybe ( MaybeT (..) )
+#if MIN_VERSION_transformers(0,6,0)
+import           Control.Monad.Trans.Maybe ( hoistMaybe )
+#endif
+import           Data.Aeson.Types ( Value, parseEither )
+import           Data.Aeson.WarningParser ( WithJSONWarnings (..) )
 #if !MIN_VERSION_rio(0,1,17)
 import           Data.Bifunctor ( bimap )
 #endif
@@ -208,7 +214,7 @@ import           Data.Monoid ( Endo (..) )
 import           Data.Time ( diffUTCTime, getCurrentTime )
 import qualified Data.Yaml as Yaml
 import           Data.Yaml.Include ( decodeFileWithWarnings )
-import           Database.Persist ( entityKey )
+import           Database.Persist.Class.PersistEntity ( entityKey )
 import           Distribution.PackageDescription
                    ( FlagName, GenericPackageDescription )
 import qualified Distribution.PackageDescription as D
@@ -217,18 +223,66 @@ import qualified Hpack
 import qualified Hpack.Config as Hpack
 import           Hpack.Error ( formatHpackError )
 import           Hpack.Yaml ( formatWarning )
-import           Network.HTTP.Download
+import           Network.HTTP.Download ( download, redownload )
 import           Pantry.Archive
-import           Pantry.Casa
-import           Pantry.HTTP
+                   ( fetchArchives, findCabalOrHpackFile, getArchive
+                   , getArchiveKey, getArchivePackage
+                   )
+import           Pantry.Casa ( casaBlobSource, casaLookupKey, casaLookupTree )
+import           Pantry.HTTP ( httpSinkChecked, parseRequest )
 import           Pantry.Hackage
-import           Pantry.Internal.AesonExtended ( Value, WithJSONWarnings (..) )
+                   ( DidUpdateOccur (..), RequireHackageIndex (..)
+                   , UsePreferredVersions (..), getHackageCabalFile
+                   , getHackagePackageVersionRevisions
+                   , getHackagePackageVersions, getHackageTarball
+                   , getHackageTarballKey, getHackageTypoCorrections
+                   , hackageIndexTarballL, htrPackage, updateHackageIndex
+                   )
 import           Pantry.Repo
+                   ( fetchRepos, fetchReposRaw, getRepo, getRepoKey, withRepo )
 import qualified Pantry.SHA256 as SHA256
-import           Pantry.Storage hiding
-                   ( TreeEntry, PackageName, Version, findOrGenerateCabalFile )
-import           Pantry.Tree
+import           Pantry.Storage
+                   ( getSnapshotCacheByHash, getSnapshotCacheId, getTreeForKey
+                   , initStorage, loadBlob, loadCachedTree
+                   , loadExposedModulePackages, loadPackageById, loadURLBlob
+                   , storeSnapshotModuleCache, storeTree, storeURLBlob
+                   , withStorage
+                   )
+import           Pantry.Tree ( rawParseGPD, unpackTree )
 import           Pantry.Types as P
+                   ( Archive (..), ArchiveLocation (..), BlobKey (..)
+                   , CabalFileInfo (..), CabalString (..), FileSize (..)
+                   , FuzzyResults (..), HackageSecurityConfig (..)
+                   , HasPantryConfig (..), HpackExecutable (..), Mismatch (..)
+                   , ModuleName, Package (..), PackageCabal (..)
+                   , PackageIdentifier (..), PackageIdentifierRevision (..)
+                   , PackageIndexConfig (..), PackageLocation (..)
+                   , PackageLocationImmutable (..), PackageMetadata (..)
+                   , PackageName, PantryConfig (..), PantryException (..)
+                   , PHpack (..), PrintWarnings (..), RawArchive (..)
+                   , RawPackageLocation (..), RawPackageLocationImmutable (..)
+                   , RawPackageMetadata (..), RawSnapshot (..)
+                   , RawSnapshotLayer (..), RawSnapshotLocation (..)
+                   , RawSnapshotPackage (..), RelFilePath (..), Repo (..)
+                   , RepoType (..), ResolvedPath (..), Revision (..)
+                   , SafeFilePath, SHA256, SimpleRepo (..), SnapName (..)
+                   , Snapshot (..), SnapshotCacheHash (..), SnapshotLayer (..)
+                   , SnapshotLocation (..), SnapshotPackage (..), Tree (..)
+                   , TreeEntry (..), TreeKey (..), Unresolved, Version
+                   , WantedCompiler (..), bsToBlobKey, cabalFileName
+                   , defaultHackageSecurityConfig, defaultSnapshotLocation
+                   , flagNameString, getGlobalHintsFile, mkSafeFilePath
+                   , moduleNameString, packageIdentifierString
+                   , packageNameString, parseFlagName, parseHackageText
+                   , parsePackageIdentifier, parsePackageIdentifierRevision
+                   , parsePackageName, parsePackageNameThrowing
+                   , parseRawSnapshotLocation, parseSnapName, parseTreeM
+                   , parseVersion, parseVersionThrowing, parseWantedCompiler
+                   , pirForHash, resolvePaths, snapshotLocation
+                   , toCabalStringMap, toRawPL, toRawPLI, toRawPM, toRawSL
+                   , toRawSnapshotLayer, unCabalStringMap, unSafeFilePath
+                   , versionString, warnMissingCabalFile
+                   )
 import           Path
                    ( Abs, Dir, File, Path, (</>), filename, parent, parseAbsDir
                    , parseRelFile, toFilePath
@@ -240,14 +294,22 @@ import           RIO.Directory ( getAppUserDataDirectory )
 import qualified RIO.FilePath as FilePath
 import qualified RIO.List as List
 import qualified RIO.Map as Map
-import           RIO.PrettyPrint
+import           RIO.PrettyPrint ( HasTerm (..) )
 import           RIO.PrettyPrint.StylesUpdate
+                   ( HasStylesUpdate (..), StylesUpdate )
 import           RIO.Process
+                   ( HasProcessContext (..), proc, runProcess_, withWorkingDir )
 import qualified RIO.Set as Set
 import           RIO.Text ( unpack )
 import qualified RIO.Text as T
 import           System.IO.Error ( isDoesNotExistError )
 
+#if !MIN_VERSION_transformers(0,6,0)
+-- | Convert a 'Maybe' computation to 'MaybeT'.
+hoistMaybe :: (Applicative m) => Maybe b -> MaybeT m b
+hoistMaybe = MaybeT . pure
+#endif
+
 decodeYaml :: FilePath -> IO (Either String ([String], Value))
 decodeYaml file = do
   bimap displayException (first formatWarnings) <$> decodeFileWithWarnings file
@@ -282,7 +344,7 @@ withPantryConfig ::
   -> Int
      -- ^ Maximum connection count
   -> CasaRepoPrefix
-     -- ^ The casa pull URL e.g. https://casa.fpcomplete.com/v1/pull.
+     -- ^ The casa pull URL e.g. https://casa.stackage.org/v1/pull.
   -> Int
      -- ^ Max casa keys to pull per request.
   -> (SnapName -> RawSnapshotLocation)
@@ -344,7 +406,7 @@ withPantryConfig' root pic he count mCasaConfig snapLoc inner = do
 --
 -- @since 0.1.1.1
 defaultCasaRepoPrefix :: CasaRepoPrefix
-defaultCasaRepoPrefix = $(thParserCasaRepo "https://casa.fpcomplete.com")
+defaultCasaRepoPrefix = $(thParserCasaRepo "https://casa.stackage.org")
 
 -- | Default max keys to pull per request.
 --
@@ -956,23 +1018,18 @@ tryLoadPackageRawViaDbOrCasa ::
   => RawPackageLocationImmutable
   -> TreeKey
   -> RIO env (Maybe Package)
-tryLoadPackageRawViaDbOrCasa rpli treeKey' = do
-  mviaDb <- tryLoadPackageRawViaLocalDb rpli treeKey'
-  case mviaDb of
-    Just package -> do
-      logDebug ("Loaded package from Pantry: " <> display rpli)
-      pure (Just package)
-    Nothing -> do
-      mCasaConfig <- view $ pantryConfigL . to pcCasaConfig
-      case mCasaConfig of
-        Nothing -> pure Nothing
-        Just _ -> do
-          mviaCasa <- tryLoadPackageRawViaCasa rpli treeKey'
-          case mviaCasa of
-            Just package -> do
-              logDebug ("Loaded package from Casa: " <> display rpli)
-              pure (Just package)
-            Nothing -> pure Nothing
+tryLoadPackageRawViaDbOrCasa rpli treeKey' = runMaybeT $
+  tryViaLocalDb <|> tryCasa
+ where
+  tryViaLocalDb = do
+    package <- MaybeT $ tryLoadPackageRawViaLocalDb rpli treeKey'
+    lift $ logDebug ("Loaded package from Pantry: " <> display rpli)
+    pure package
+  tryCasa = do
+    void $ MaybeT $ view $ pantryConfigL . to pcCasaConfig
+    package <- MaybeT $ tryLoadPackageRawViaCasa rpli treeKey'
+    lift $ logDebug ("Loaded package from Casa: " <> display rpli)
+    pure package
 
 -- | Maybe load the package from Casa.
 tryLoadPackageRawViaCasa ::
@@ -980,23 +1037,20 @@ tryLoadPackageRawViaCasa ::
   => RawPackageLocationImmutable
   -> TreeKey
   -> RIO env (Maybe Package)
-tryLoadPackageRawViaCasa rlpi treeKey' = do
-  mtreePair <- casaLookupTree treeKey'
-  case mtreePair of
-    Nothing -> pure Nothing
-    Just (treeKey'', _tree) -> do
-      fetchTreeKeys [rlpi]
-      mdb <- tryLoadPackageRawViaLocalDb rlpi treeKey''
-      case mdb of
-        Nothing -> do
-          logWarn
-            ("Did not find tree key in DB after pulling it from Casa: " <>
-             display treeKey'' <>
-             " (for " <>
-             display rlpi <>
-             ")")
-          pure Nothing
-        Just package -> pure (Just package)
+tryLoadPackageRawViaCasa rlpi treeKey' = runMaybeT $ do
+  (treeKey'', _) <- MaybeT $ casaLookupTree treeKey'
+  lift $ fetchTreeKeys [rlpi]
+  tryViaLocalDb treeKey'' <|> warn treeKey''
+ where
+  tryViaLocalDb = MaybeT . tryLoadPackageRawViaLocalDb rlpi
+  warn treeKey'' = do
+    lift $ logWarn $
+         "Did not find tree key in DB after pulling it from Casa: "
+      <> display treeKey''
+      <> " (for "
+      <> display rlpi
+      <> ")"
+    empty
 
 -- | Maybe load the package from the local database.
 tryLoadPackageRawViaLocalDb ::
@@ -1004,12 +1058,9 @@ tryLoadPackageRawViaLocalDb ::
   => RawPackageLocationImmutable
   -> TreeKey
   -> RIO env (Maybe Package)
-tryLoadPackageRawViaLocalDb rlpi treeKey' = do
-  mtreeEntity <- withStorage (getTreeForKey treeKey')
-  case mtreeEntity of
-    Nothing -> pure Nothing
-    Just treeId ->
-      fmap Just (withStorage (loadPackageById rlpi (entityKey treeId)))
+tryLoadPackageRawViaLocalDb rlpi treeKey' = runMaybeT $ do
+  treeId <- MaybeT $ withStorage (getTreeForKey treeKey')
+  lift $ withStorage (loadPackageById rlpi (entityKey treeId))
 
 -- | Complete package location, plus whether the package has a cabal file. This
 -- is relevant to reproducibility, see
@@ -1483,13 +1534,13 @@ cachedSnapshotCompletePackageLocation ::
   => Map RawPackageLocationImmutable PackageLocationImmutable
   -> RawPackageLocationImmutable
   -> RIO env (Maybe PackageLocationImmutable)
-cachedSnapshotCompletePackageLocation cachePackages rpli = do
-  let xs = Map.lookup rpli cachePackages
-  case xs of
-    Nothing -> do
-      cpl <- completePackageLocation rpli
-      pure $ if cplHasCabalFile cpl then Just (cplComplete cpl) else Nothing
-    Just x -> pure $ Just x
+cachedSnapshotCompletePackageLocation cachePackages rpli = runMaybeT $
+  tryCache <|> tryCpl
+ where
+  tryCache = hoistMaybe $ Map.lookup rpli cachePackages
+  tryCpl = do
+    cpl <- lift $ completePackageLocation rpli
+    if cplHasCabalFile cpl then pure (cplComplete cpl) else empty
 
 -- | Add more packages to a snapshot completing their locations if needed
 --
@@ -1782,7 +1833,31 @@ getTreeKey (PLIRepo _ pm) = pmTreeKey pm
 -- using 'withPantryConfig' or 'withPantryConfig'' directly. Uses basically sane
 -- settings, like sharing a pantry directory with Stack.
 --
--- You can use 'runPantryApp' to use this.
+-- You can use 'runPantryApp' to use this. A simple example is:
+--
+-- > {-# LANGUAGE OverloadedStrings #-}
+-- >
+-- > module Main (main) where
+-- >
+-- > -- From package Cabal-syntax
+-- > import Distribution.Types.Version ( mkVersion )
+-- > -- From package pantry
+-- > import Pantry
+-- >          ( CabalFileInfo (..), PackageIdentifierRevision (..), PantryApp
+-- >          , RawPackageLocationImmutable (..), loadPackageRaw, runPantryApp
+-- >          )
+-- > -- From package rio
+-- > import RIO ( RIO, liftIO )
+-- >
+-- > main :: IO ()
+-- > main = runPantryApp myPantryApp
+-- >
+-- > myPantryApp :: RIO PantryApp ()
+-- > myPantryApp = loadPackageRaw baseLocation >>= liftIO . print
+-- >  where
+-- >   baseVersion = mkVersion [4, 19, 0, 0]
+-- >   basePkgId = PackageIdentifierRevision "base" baseVersion CFILatest
+-- >   baseLocation = RPLIHackage basePkgId Nothing
 --
 -- @since 0.1.0.0
 data PantryApp = PantryApp
