@@ -15,6 +15,9 @@ module Pantry.Archive
 
 import qualified Codec.Archive.Zip as Zip
 import           Conduit
+                   ( ConduitT, (.|), runConduit, sinkHandle, sinkList
+                   , sourceHandle, sourceLazy, withSourceFile
+                   )
 import           Data.Bits ( (.&.), shiftR )
 import qualified Data.Conduit.Tar as Tar
 import           Data.Conduit.Zlib ( ungzip )
@@ -22,19 +25,34 @@ import qualified Data.Digest.CRC32 as CRC32
 import           Distribution.PackageDescription ( package, packageDescription )
 import qualified Hpack.Config as Hpack
 import           Pantry.HPack ( hpackVersion )
-import           Pantry.HTTP
+import           Pantry.HTTP ( httpSinkChecked )
 import           Pantry.Internal ( makeTarRelative, normalizeParents )
 import qualified Pantry.SHA256 as SHA256
-import           Pantry.Storage hiding
-                   ( Tree, TreeEntry, findOrGenerateCabalFile )
-import           Pantry.Tree
+import           Pantry.Storage
+                   ( BlobId, CachedTree (..), TreeId, hpackToCabal
+                   , loadArchiveCache, loadBlob, loadCabalBlobKey
+                   , loadCachedTree, loadPackageById, storeArchiveCache
+                   , storeBlob, storeHPack, storeTree, unCachedTree, withStorage
+                   )
+import           Pantry.Tree ( rawParseGPD )
 import           Pantry.Types
+                   ( Archive, ArchiveLocation (..), BlobKey, BuildFile (..)
+                   , FileSize (..), FileType (..), HasPantryConfig
+                   , Mismatch (..), Package (..), PackageCabal (..)
+                   , PackageIdentifier (..), PackageMetadata (..)
+                   , PantryException (..), PHpack (..), RawArchive (..)
+                   , RawPackageLocationImmutable (..), RawPackageMetadata (..)
+                   , ResolvedPath (..), SHA256, Tree (..), TreeEntry (..)
+                   , TreeKey, cabalFileName, hpackSafeFilePath, mkSafeFilePath
+                   , toRawArchive, toRawPM, unSafeFilePath
+                   )
 import           Path ( toFilePath )
+import           Path.IO ( doesFileExist )
 import           RIO
 import qualified RIO.ByteString.Lazy as BL
 import qualified RIO.List as List
 import qualified RIO.Map as Map
-import           RIO.Process
+import           RIO.Process ( HasProcessContext )
 import qualified RIO.Set as Set
 import qualified RIO.Text as T
 import qualified RIO.Text.Partial as T
@@ -219,7 +237,8 @@ checkPackageMetadata pl pm pa = do
    in if and tests then Right pa else Left err
 
 -- | Provide a local file with the contents of the archive, regardless of where
--- it comes from. Perform SHA256 and file size validation if downloading.
+-- it comes from. If not downloading, checks that the archive file exists.
+-- Performs SHA256 and file size validation.
 withArchiveLoc ::
      HasLogFunc env
   => RawArchive
@@ -228,6 +247,9 @@ withArchiveLoc ::
 withArchiveLoc (RawArchive (ALFilePath resolved) msha msize _subdir) f = do
   let abs' = resolvedAbsolute resolved
       fp = toFilePath abs'
+  archiveExists <- doesFileExist abs'
+  unless archiveExists $
+    throwIO $ LocalNoArchiveFileFound abs'
   (sha, size) <- withBinaryFile fp ReadMode $ \h -> do
     size <- FileSize . fromIntegral <$> hFileSize h
     for_ msize $ \size' ->
@@ -288,7 +310,8 @@ foldArchive loc fp ATTar accum f =
   withSourceFile fp $ \src -> runConduit $ src .| foldTar loc accum f
 foldArchive loc fp ATZip accum0 f = withBinaryFile fp ReadMode $ \h -> do
   let go accum entry = do
-        let me = MetaEntry (Zip.eRelativePath entry) met
+        let normalizedRelPath = removeInitialDotSlash $ Zip.eRelativePath entry
+            me = MetaEntry normalizedRelPath met
             met = fromMaybe METNormal $ do
               let modes = shiftR (Zip.eExternalFileAttributes entry) 16
               guard $ Zip.eVersionMadeBy entry .&. 0xFF00 == 0x0300
@@ -345,7 +368,7 @@ foldTar loc accum0 f = do
         _ -> throwIO exc
     pure $
       (\met -> MetaEntry
-        { mePath = Tar.getFileInfoPath fi
+        { mePath = removeInitialDotSlash . Tar.getFileInfoPath $ fi
         , meType = met
         })
       <$> mmet
@@ -356,6 +379,10 @@ data SimpleEntry = SimpleEntry
   }
   deriving Show
 
+removeInitialDotSlash :: FilePath -> FilePath
+removeInitialDotSlash filename =
+  fromMaybe filename $ List.stripPrefix "./" filename
+
 -- | Attempt to parse the contents of the given archive in the given subdir into
 -- a 'Tree'. This will not consult any caches. It will ensure that:
 --
@@ -365,22 +392,28 @@ data SimpleEntry = SimpleEntry
 --
 -- * The name inside the cabal file matches the name of the cabal file itself
 parseArchive ::
-     (HasPantryConfig env, HasLogFunc env, HasProcessContext env)
+     forall env. (HasPantryConfig env, HasLogFunc env, HasProcessContext env)
   => RawPackageLocationImmutable
   -> RawArchive
   -> FilePath -- ^ file holding the archive
   -> RIO env (Package, CachedTree)
 parseArchive rpli archive fp = do
   let loc = raLocation archive
+      archiveTypes :: [ArchiveType]
+      archiveTypes = [minBound .. maxBound]
+      getFiles :: [ArchiveType] -> RIO env (ArchiveType, Map FilePath MetaEntry)
       getFiles [] = throwIO $ UnknownArchiveType loc
       getFiles (at:ats) = do
-        eres <- tryAny $ foldArchive loc fp at id $ \m me -> pure $ m . (me:)
+        eres <- tryAny $
+          -- foldArchive normalises filepaths in archives that begin with ./
+          foldArchive loc fp at id $ \m me -> pure $ m . (me:)
         case eres of
           Left e -> do
             logDebug $ "parseArchive of " <> display at <> ": " <> displayShow e
             getFiles ats
-          Right files -> pure (at, Map.fromList $ map (mePath &&& id) $ files [])
-  (at :: ArchiveType, files :: Map FilePath MetaEntry) <- getFiles [minBound..maxBound]
+          Right files ->
+            pure (at, Map.fromList $ map (mePath &&& id) $ files [])
+  (at, files) <- getFiles archiveTypes
   let toSimple :: FilePath -> MetaEntry -> Either String (Map FilePath SimpleEntry)
       toSimple key me =
         case meType me of
@@ -439,7 +472,8 @@ parseArchive rpli archive fp = do
                           ++ "See https://github.com/commercialhaskell/stack/issues/4579 \
                              \for further information."
                   pairs ->
-                    fmap fold $ for pairs $ \(suffix, me') -> toSimple (key ++ '/' : suffix) me'
+                    fmap fold $ for pairs $ \(suffix, me') ->
+                      toSimple (key ++ '/' : suffix) me'
               Just me' ->
                 case meType me' of
                   METNormal ->
@@ -466,7 +500,7 @@ parseArchive rpli archive fp = do
         Left e -> throwIO $ UnsupportedTarball loc $ T.pack e
         Right safeFiles -> do
           let toSave = Set.fromList $ map (seSource . snd) safeFiles
-          (blobs :: Map FilePath (BlobKey, BlobId))  <-
+          (blobs :: Map FilePath (BlobKey, BlobId)) <-
             foldArchive loc fp at mempty $ \m me ->
               if mePath me `Set.member` toSave
                 then do
@@ -474,12 +508,13 @@ parseArchive rpli archive fp = do
                   (blobId, blobKey) <- lift $ withStorage $ storeBlob bs
                   pure $ Map.insert (mePath me) (blobKey, blobId) m
                 else pure m
-          tree :: CachedTree <- fmap (CachedTreeMap . Map.fromList) $ for safeFiles $ \(sfp, se) ->
-            case Map.lookup (seSource se) blobs of
-              Nothing ->
-                error $ "Impossible: blob not found for: " ++ seSource se
-              Just (blobKey, blobId) ->
-                pure (sfp, (TreeEntry blobKey (seType se), blobId))
+          tree :: CachedTree <-
+            fmap (CachedTreeMap . Map.fromList) $ for safeFiles $ \(sfp, se) ->
+              case Map.lookup (removeInitialDotSlash . seSource $ se) blobs of
+                Nothing ->
+                  error $ "Impossible: blob not found for: " ++ seSource se
+                Just (blobKey, blobId) ->
+                  pure (sfp, (TreeEntry blobKey (seType se), blobId))
           -- parse the cabal file and ensure it has the right name
           buildFile <- findCabalOrHpackFile rpli $ unCachedTree tree
           (buildFilePath, buildFileBlobKey, buildFileEntry) <- case buildFile of
