@@ -1,4 +1,5 @@
 {-# LANGUAGE NoImplicitPrelude   #-}
+{-# LANGUAGE CPP                 #-}
 {-# LANGUAGE LambdaCase          #-}
 {-# LANGUAGE OverloadedStrings   #-}
 {-# LANGUAGE RecordWildCards     #-}
@@ -15,10 +16,21 @@ module Pantry.Repo
   , withRepo
   ) where
 
-import           Database.Persist ( Entity (..) )
-import           Pantry.Archive
-import           Pantry.Storage hiding ( findOrGenerateCabalFile )
+import           Database.Persist.Class.PersistEntity ( Entity (..) )
+import           Pantry.Archive ( getArchivePackage )
+import           Pantry.Storage
+                   ( getTreeForKey, loadPackageById, loadRepoCache
+                   , storeRepoCache, withStorage
+                   )
 import           Pantry.Types
+                   ( AggregateRepo (..), ArchiveLocation (..), HasPantryConfig
+                   , Package (..), PackageMetadata (..), PantryException (..)
+                   , RawArchive (..), RawPackageLocationImmutable (..)
+                   , RawPackageMetadata (..), RelFilePath (..), Repo (..)
+                   , RepoType (..), ResolvedPath (..), SimpleRepo (..)
+                   , TreeKey (..), arToSimpleRepo, rToSimpleRepo
+                   , toAggregateRepos, toRawPM
+                   )
 import           Path.IO ( resolveFile' )
 import           RIO
 import           RIO.ByteString ( isInfixOf )
@@ -27,8 +39,16 @@ import           RIO.Directory ( doesDirectoryExist )
 import           RIO.FilePath ( (</>) )
 import qualified RIO.Map as Map
 import           RIO.Process
+                   ( ExitCodeException (..), HasProcessContext, proc
+                   , readProcess, readProcess_, withModifyEnvVars
+                   , withWorkingDir
+                   )
 import qualified RIO.Text as T
+#if MIN_VERSION_ansi_terminal(1, 0, 2)
+import           System.Console.ANSI ( hNowSupportsANSI )
+#else
 import           System.Console.ANSI ( hSupportsANSIWithoutEmulation )
+#endif
 import           System.IsWindows ( osIsWindows )
 
 data TarType = Gnu | Bsd
@@ -223,13 +243,19 @@ archiveSubmodules tarball = do
           then " --force-local "
           else mempty
   case tarType of
-    Gnu -> runGitCommand
-      [ "submodule"
-      , "foreach"
-      , "--recursive"
-      , "git -c core.autocrlf=false archive --prefix=$displaypath/ -o bar.tar HEAD; "
-        <> "tar" <> forceLocal <> " -Af " <> tarball <> " bar.tar"
-      ]
+    Gnu -> do
+      -- Single quotation marks are required around tarball because otherwise,
+      -- in the foreach environment, the \ character in absolute paths on
+      -- Windows will be interpreted as escaping the following character.
+      let foreachCommand =
+            "git -c core.autocrlf=false archive --prefix=$displaypath/ -o bar.tar HEAD; "
+            <> "tar" <> forceLocal <> " -Af '" <> tarball <> "' bar.tar"
+      runGitCommand
+        [ "submodule"
+        , "foreach"
+        , "--recursive"
+        , foreachCommand
+        ]
     Bsd -> runGitCommand
       [ "submodule"
       , "foreach"
@@ -296,7 +322,12 @@ withRepo sr@SimpleRepo{..} action =
           -- that command clears, but does not then restore, the
           -- ENABLE_VIRTUAL_TERMINAL_PROCESSING flag for native terminals. The
           -- following hack re-enables the lost ANSI-capability.
-          when osIsWindows $ void $ liftIO $ hSupportsANSIWithoutEmulation stdout
+          when osIsWindows $ void $ liftIO $
+#if MIN_VERSION_ansi_terminal(1, 0, 2)
+            hNowSupportsANSI stdout
+#else
+            hSupportsANSIWithoutEmulation stdout
+#endif
     logInfo $ "Cloning " <> display sRepoCommit <> " from " <> display sRepoUrl
     runCommand ["clone", repoUrl, dir]
     fixANSIForWindows
