@@ -8,10 +8,16 @@ module Pantry.SQLite
   , initStorage
   ) where
 
+import           Control.Concurrent.Companion
+                   ( Companion, onCompanionDone, withCompanion )
+import           Database.Persist.Sql ( runSqlConn )
+import           Database.Persist.Sql.Migration
+                   ( Migration, runMigrationSilent )
 import           Database.Persist.Sqlite
-import           Pantry.Internal.Companion
-import           Pantry.Types
-                   ( PantryException (MigrationFailure), Storage (..) )
+                   ( extraPragmas, fkEnabled, mkSqliteConnectionInfo
+                   , walEnabled, withSqliteConnInfo
+                   )
+import           Pantry.Types ( PantryException (..), Storage (..) )
 import           Path ( Abs, File, Path, parent, toFilePath )
 import           Path.IO ( ensureDir )
 import           RIO hiding ( FilePath )
@@ -21,10 +27,10 @@ import           System.FileLock
 
 initStorage ::
      HasLogFunc env
-  => Text
-  -> Migration
-  -> Path Abs File -- ^ storage file
-  -> (Storage -> RIO env a)
+  => Text -- ^ Database description, for lock messages.
+  -> Migration -- ^ Initial migration.
+  -> Path Abs File -- ^ SQLite database file.
+  -> (Storage -> RIO env a) -- ^ What to do with the initialised 'Storage'.
   -> RIO env a
 initStorage description migration fp inner = do
   ensureDir $ parent fp
@@ -75,7 +81,7 @@ initStorage description migration fp inner = do
 -- above.
 withWriteLock ::
      HasLogFunc env
-  => Utf8Builder -- ^ database description, for lock messages
+  => Utf8Builder -- ^ Database description, for lock messages
   -> Path Abs File -- ^ SQLite database file
   -> RIO env a
   -> RIO env a
